/*
 * File:  ndef.c
 * Use:   higher level ntag routines
 * Autjor:  Jeff Seaman
 * Project: ntage & ble demo
 * Processor: nRF52840
 * SDK:       nRF5_SDK_16.0.0
 */
#include <stdio.h>
#include "nrf_delay.h"
#include "ntag.h"

/**************************************************************
 * variables
 **************************************************************/
unsigned char tlv_buff[64];
uint8_t ndef_write_buffer[16];

/*************************************************************
 * local defines
 *************************************************************/
#define TLV_TAG_FIELD 0
#define TLV_LENGTH_FIELD 1
#define NDEF_HEADER_BYTE 2
#define NDEF_TYPE_LENGTH 3
#define NDEF_PAYLOAD_LENGTH 4
#define NDEF_RECORD_TYPE 5
#define NDEF_STATUS 6
#define NDEF_LANG_CODE_1 7
#define NDEF_LANG_CODE_2 8
#define NDEF_TEXT_START 9

#define NDEF_WWW_URI 6
#define NDEF_WWW_START 7

/**************************************************************
 * Formats the ntag for use with ndef messages
 **************************************************************/
void format_ndeftag(){
	unsigned char formatbuffer[16];
	uint8_t format = 0;
	read_ntag(0,formatbuffer);
	if(formatbuffer[12] != 0xE1) format = 1;
	if(formatbuffer[12] != 0x10) format = 1;
	if(formatbuffer[12] != 0x6D) format = 1;
	if(formatbuffer[12] != 0x00) format = 1;

	if(format){
		formatbuffer[0] =0xAA;
		formatbuffer[8] = 0x44;
		formatbuffer[9] = 0x00;
		formatbuffer[10] = 0x00;
		formatbuffer[11] = 0x00;

		formatbuffer[12] = 0xE1;
		formatbuffer[13] = 0x10;
		formatbuffer[14] = 0x6D;
		formatbuffer[15] = 0x00;
		write_ntag(0,formatbuffer);
	}
}
/**************************************************************
 * writes a ndef text message
 * @param buff the string to be encoded
 **************************************************************/
void write_ndef_text(char * buff ){
	// copy the string into the ndef payload
	unsigned char	 len;
	len = strlen(buff);
	for (int i = 0; i < len; ++i) {
		tlv_buff[i + NDEF_TEXT_START ] = buff[i];
	}
	// create the pay load header

	tlv_buff[NDEF_STATUS] = 0x02;			// UNICODE - 8 format and 2 LANGUEGE code bytes
	tlv_buff[NDEF_LANG_CODE_1] = 'e';		// we are using English
	tlv_buff[NDEF_LANG_CODE_2] = 'n';
	// create the NDEF header
	tlv_buff[NDEF_HEADER_BYTE] = 0xD1;	// single short record of well known type
	tlv_buff[NDEF_TYPE_LENGTH] = 1;		// we use a single message type value
												// length of the text plus the pay load header
	tlv_buff[NDEF_PAYLOAD_LENGTH] = len + 3;
	tlv_buff[NDEF_RECORD_TYPE] = 'T';		// Text message

	// create the TLV header
	tlv_buff[TLV_TAG_FIELD] = 0x03;		// Block contains an NDEF message
	tlv_buff[TLV_LENGTH_FIELD] = tlv_buff[NDEF_PAYLOAD_LENGTH] + 4;

	// write the end of TLF marker 0x3F in last space of message
	uint8_t len_tlv = tlv_buff[TLV_LENGTH_FIELD]+2;
	tlv_buff[len_tlv+2] = 0xFE;

	// write the message chunks to the NTAG
	unsigned char write_buffer[16];
	len = 0;
	unsigned char block = 1;
	while  ( len < len_tlv){
		for(int k =0; k < 16; ++k){
			write_buffer[k] = tlv_buff[len + k];
		}

		write_ntag(block,write_buffer);
		delay_20ms();
		++ block;
		len = len + 16;
	}
}

/***************************************************
 * write out a web address for the ndef message
 * @param buff the web addres to be writen
 ***************************************************/
void write_ndef_www(char * buff ){
	// copy the string into the ndef payload
	memset(tlv_buff,0xff,64);
		unsigned char	 len;
		len = strlen(buff);
		for (int i = 0; i < len; ++i) {
			tlv_buff[i + NDEF_WWW_START ] = buff[i];
		}
		// create the NDEF payload header
		tlv_buff[NDEF_PAYLOAD_LENGTH] = len+1;
		tlv_buff[NDEF_RECORD_TYPE] = 'U';		// Text message
		tlv_buff[NDEF_WWW_URI]= 3;				// http://   address
		// create the ndef header
		tlv_buff[NDEF_HEADER_BYTE] = 0xD1;	// single short record of well known type
		tlv_buff[NDEF_TYPE_LENGTH] = 1;		// we use a single message type value
		// create the TLV header
			tlv_buff[TLV_TAG_FIELD] = 0x03;		// Block contains an NDEF message
			tlv_buff[TLV_LENGTH_FIELD] = tlv_buff[NDEF_PAYLOAD_LENGTH] + 4;
	// write the end of TLF marker 0x3F in last space of message
		uint8_t len_tlv = tlv_buff[TLV_LENGTH_FIELD]+2;
		tlv_buff[len_tlv] = 0xFE;
	// write the message chunks to the NTAG
		unsigned char write_buffer[16];
		len = 0;
		unsigned char block = 1;
		while  ( len <= len_tlv){
			for(int k =0; k < 16; ++k){
				write_buffer[k] = tlv_buff[len + k];
			}
			write_ntag(block,write_buffer);
		delay_20ms();
			++ block;
			len = len + 16;
		}
}










